/* 
 * Author    : Eren, Kazim Anil
 * Crea Date : June 29th, 2018
 * Objective : Creating events based on outgoing fund transfers. (TL Based)
 * Modified  : Fixed event names since events are based on TL events. - June 29th, 2018.
 */

/* 0. Time Variable Selection
SELECT	y.cmusno,
		x.*
FROM	(
		SELECT	sube, gon_hesno, tar, hvl_tar, tar, tut 
		FROM 	staging.pov_dwf_havale
		) x 
JOIN	(
SELECT	sube, hesno, cmusno
FROM	staging.pov_ybs_hesap_ortak_musteri
) y
	ON	x.sube = y.sube
	AND	x.gon_hesno = y.hesno
WHERE	CAST(tar as DATE) <> CAST(tar as DATE) 
*/

/* 1.Preparing Raw Data */

DROP TABLE IF EXISTS komtas.dwf;
CREATE TABLE komtas.dwf
DISTRIBUTE BY HASH (gon_hesno)
AS 
( 
SELECT	sube, gon_hesno, CAST(tar AS DATE) as tarih, sum(tut) as tutar
FROM	staging.pov_dwf_havale
WHERE 	HVL_TUR = 1 
	AND TRANSFER_SEKLI = 3
	AND	parakod = 'TL'
	AND gon_hesno IS NOT NULL
	AND tar BETWEEN (SELECT start_date FROM onboarding.control_path) AND
							(SELECT end_date   FROM onboarding.control_path)
GROUP	BY sube, gon_hesno, CAST(tar AS DATE) 
);

DROP TABLE IF EXISTS komtas.acct;
CREATE TABLE komtas.acct 
DISTRIBUTE BY HASH (cmusno)
as 
(
SELECT	sube, hesno, cmusno
FROM	staging.pov_ybs_hesap_ortak_musteri
);


DROP TABLE IF EXISTS komtas.ham;
CREATE TABLE komtas.ham 
DISTRIBUTE BY HASH (cmusno)
AS
(
SELECT	cmusno, 
		tarih, extract(year from tarih) as yil, extract(month from tarih) as ay, 
		tutar
FROM	komtas.dwf dwf
JOIN	komtas.acct acct
	ON	dwf.gon_hesno = acct.hesno
	AND	dwf.sube = acct.sube
);

DROP TABLE komtas.dwf;  -- Not the best way but I could not get TEMP/WITH to work. 
DROP TABLE komtas.acct; -- Not the best way but I could not get TEMP/WITH to work.

/* 2. Manipulating Data for the Flags */
DROP TABLE IF EXISTS komtas.anomali_ham;
CREATE TABLE komtas.anomali_ham
DISTRIBUTE BY HASH (cmusno)
AS
(
SELECT	cum_sum.* 
		, aylik.gecmis_3ay_ort
		, CASE WHEN cum_sum >= 1.2 * aylik.gecmis_3ay_ort AND cum_sum < 1.5 * aylik.gecmis_3ay_ort THEN 1 ELSE 0 END AS anomali_20flag
		, CASE WHEN cum_sum >= 1.5 * aylik.gecmis_3ay_ort AND cum_sum < 2.0 * aylik.gecmis_3ay_ort THEN 1 ELSE 0 END AS anomali_50flag
		, CASE WHEN cum_sum >= 2.0 * aylik.gecmis_3ay_ort THEN 1 ELSE 0 END AS anomali_100flag
FROM	(
	SELECT	*, 
			SUM(tutar) OVER (PARTITION 	BY cmusno, yil, ay 
							 ORDER 		BY tarih) as cum_sum
	FROM	komtas.ham
	ORDER	BY cmusno, yil, ay, tarih
	) cum_sum
JOIN	(
	SELECT	cmusno, yil, ay, aylik_tutar,
			ROUND(AVG(aylik_tutar) OVER (PARTITION BY cmusno ORDER BY yil, ay ROWS BETWEEN 3 PRECEDING AND 1 PRECEDING), 2) as gecmis_3ay_ort
	FROM	(
		SELECT	cmusno, yil, ay, sum(tutar) as aylik_tutar
		FROM	komtas.ham
		GROUP	BY cmusno, yil, ay
			) alia
		) aylik
	ON	cum_sum.cmusno = aylik.cmusno
	AND	cum_sum.yil    = aylik.yil
	AND cum_sum.ay     = aylik.ay
WHERE 	CASE WHEN cum_sum >= 1.2 * aylik.gecmis_3ay_ort AND cum_sum < 1.5 * aylik.gecmis_3ay_ort THEN 1 ELSE 0 END = 1
	OR	CASE WHEN cum_sum >= 1.5 * aylik.gecmis_3ay_ort AND cum_sum < 2.0 * aylik.gecmis_3ay_ort THEN 1 ELSE 0 END = 1
	OR	CASE WHEN cum_sum >= 2.0 * aylik.gecmis_3ay_ort THEN 1 ELSE 0 END = 1
);

/* 3. Preparing Flags / Events for goker.event_list */

/* Deleting Previous Inserts */
--DELETE	
--FROM	goker.event_list
--WHERE	event_id IN ('26010', '26011', '26012');

/* 3.2. Inserting AnomalyTL20 flags */
INSERT	
INTO	onboarding.event_list
SELECT	cmusno
		, '26010' as event_id
		, event_date
		, 'FUND_TRANSFER' as domain_type
		, 'OutgoingFundTrf' as Event1
		, 'AnomalyTL20pct' as Event2
		, '' as Event3
		, '' as chn_name
FROM	(
SELECT	cmusno, CAST(min(tarih) as TIMESTAMP) as event_date
FROM	(SELECT * FROM komtas.anomali_ham WHERE anomali_20flag = 1 ORDER BY cmusno, yil, ay, tarih) a20
GROUP	BY cmusno, yil, ay, anomali_20flag
) a20;
--STATEMENT 11: Insert Statement completed. 575409  rows processed. 

/* 3.3. Inserting AnomalyTL50 flags */
INSERT	
INTO	onboarding.event_list
SELECT	cmusno
		, '26011' as event_id
		, event_date
		, 'FUND_TRANSFER' as domain_type
		, 'OutgoingFundTrf' as Event1
		, 'AnomalyTL50pct' as Event2
		, '' as Event3
		, '' as chn_name
FROM	(
SELECT	cmusno, CAST(min(tarih) as TIMESTAMP) as event_date
FROM	(SELECT * FROM komtas.anomali_ham WHERE anomali_50flag = 1 ORDER BY cmusno, yil, ay, tarih) a50
GROUP	BY cmusno, yil, ay, anomali_50flag
) a50;
--STATEMENT 12: Insert Statement completed. 479741  rows processed.

/* 3.4. Inserting AnomalyTL100 flags */
INSERT	
INTO	onboarding.event_list
SELECT	cmusno
		, '26012' as event_id
		, event_date
		, 'FUND_TRANSFER' as domain_type
		, 'OutgoingFundTrf' as Event1
		, 'AnomalyTL100pct' as Event2
		, '' as Event3
		, '' as chn_name
FROM	(
SELECT	cmusno, CAST(min(tarih) as TIMESTAMP) as event_date
FROM	(SELECT * FROM komtas.anomali_ham WHERE anomali_100flag = 1 ORDER BY cmusno, yil, ay, tarih) a100
GROUP	BY cmusno, yil, ay, anomali_100flag
) a100;
--STATEMENT 13: Insert Statement completed. 767851  rows processed. 

DROP TABLE komtas.anomali_ham;
DROP TABLE komtas.ham;