/* 
 * Author    : Eren, Kazim Anil
 * Crea Date : June 29th, 2018
 * Objective : Creating events based on outgoing fund transfers. (TL Based, Using last three months' maximum transfer values.)
 * Modified  : NO
 */

/* 1. Raw Table Creation Process */

DROP TABLE IF EXISTS komtas.dwf;
CREATE TABLE komtas.dwf
DISTRIBUTE BY HASH (gon_hesno)
AS 
( 
SELECT	sube, 
		gon_hesno,
		tar 					as tarih,
		extract(year from tar) 	as yil, 
		extract(month from tar)	as ay, 
		tut								as tutar
FROM	staging.pov_dwf_havale
WHERE 	HVL_TUR = 1 
	AND TRANSFER_SEKLI = 3
	AND	parakod = 'TL'
	AND gon_hesno IS NOT NULL
	AND tar BETWEEN (SELECT start_date FROM onboarding.control_path) AND
							(SELECT end_date   FROM onboarding.control_path)
);

DROP TABLE IF EXISTS komtas.acct;
CREATE TABLE komtas.acct 
DISTRIBUTE BY HASH (cmusno)
as 
(
SELECT	sube, hesno, cmusno
FROM	staging.pov_ybs_hesap_ortak_musteri
);

/* 1.1. Reference Table is created since a customer does not have to make transfers each month */ 
DROP TABLE IF EXISTS komtas.ref_table;
CREATE TABLE komtas.ref_table
DISTRIBUTE BY HASH (cmusno)
as
(
SELECT	*
FROM
(
SELECT	cmusno
FROM	komtas.dwf dwf
JOIN	komtas.acct acct
	ON	dwf.gon_hesno = acct.hesno
	AND	dwf.sube = acct.sube
GROUP	BY cmusno
) cmus
JOIN
(
SELECT	yil, ay
FROM	komtas.dwf dwf
GROUP	BY yil, ay 
) tarih
ON 1=1
ORDER BY cmusno, yil, ay
);

/* Aylik means Monthly since the data is aggregated by year-months in this step*/
DROP TABLE IF EXISTS komtas.aylik;
CREATE TABLE komtas.aylik
DISTRIBUTE BY HASH (cmusno)
AS
(
SELECT	*
		,LAG(tutar, 1, NULL) OVER (PARTITION BY cmusno ORDER BY yil, ay) as tutar1 -- no transfer previous month means maximum is 0.
		,LAG(tutar, 2, NULL) OVER (PARTITION BY cmusno ORDER BY yil, ay) as tutar2 -- no transfer previous month means maximum is 0.
		,LAG(tutar, 3, NULL) OVER (PARTITION BY cmusno ORDER BY yil, ay) as tutar3 -- no transfer previous month means maximum is 0.
FROM	(
	SELECT	rt.*, tutar
	FROM	komtas.ref_table rt
	LEFT JOIN	(
		SELECT	cmusno, yil, ay, max(tutar) as tutar
		FROM	komtas.dwf dwf
		JOIN	komtas.acct acct
			ON	dwf.gon_hesno = acct.hesno
			AND	dwf.sube = acct.sube
		GROUP	BY cmusno, yil, ay
				) qr
	ON	rt.cmusno = qr.cmusno
	AND	rt.yil    = qr.yil
	AND rt.ay     = qr.ay
		)sqry
ORDER	BY cmusno, yil, ay
);

/* Ham means Raw. We will create the flags on this table */
DROP TABLE IF EXISTS komtas.ham;
CREATE TABLE komtas.ham 
DISTRIBUTE BY HASH (cmusno)
AS
(
SELECT	iq.*,
		CASE WHEN TUTAR1 IS NULL AND TUTAR2 IS NULL AND TUTAR3 IS NULL THEN NULL
			 ELSE ROUND((COALESCE(tutar1, 0) + COALESCE(tutar2, 0) + COALESCE(tutar3,0)) / ( 
			 	(CASE WHEN TUTAR1 IS NULL THEN 0 ELSE 1 END) +
			 	(CASE WHEN TUTAR2 IS NULL THEN 0 ELSE 1 END) +
			 	(CASE WHEN TUTAR3 IS NULL THEN 0 ELSE 1 END) ), 2)
			 END AS esik -- esik means threshold / long code to get the average correctly.	
FROM	(
SELECT	cmusno, tarih, yil, ay, tutar
FROM	komtas.dwf dwf
JOIN	komtas.acct acct
	ON	dwf.gon_hesno = acct.hesno
	AND	dwf.sube = acct.sube
		) iq -- inner query
JOIN	komtas.aylik ay
	ON	ay.cmusno = iq.cmusno
	AND	ay.yil    = iq.yil
	AND ay.ay	  = iq.ay
ORDER	BY cmusno, yil, ay
);

DROP TABLE komtas.dwf;       -- Not the best practice but I could not get TEMP/WITH to work. 
DROP TABLE komtas.acct;      -- Not the best practice but I could not get TEMP/WITH to work.
DROP TABLE komtas.ref_table; -- Not the best practice but I could not get TEMP/WITH to work.


/* 2. Preparing Flags / Events for goker.event_list */

/* 2.1. Deleting Previous Inserts */
--DELETE	
--FROM	goker.event_list
--WHERE	event_id IN ('26013', '26014', '26015');

/* 2.2. Inserting AnomalyTL20 flags */ 
INSERT	
INTO	onboarding.event_list
SELECT	cmusno
		, '26013' 				as event_id
		, tarih 				as event_date
		, 'FUND_TRANSFER' 		as domain_type
		, 'OutgoingFundTrf' 	as Event1
		, 'AnomalyTL20pctMax' 	as Event2
		, '' 					as Event3
		, '' 					as chn_name
FROM	(
	SELECT * 
	FROM komtas.ham 
	WHERE yil*ay <> 2015*10 
	AND tutar >= 1.2 * esik
	AND tutar <  1.5 * esik
		) a20max;
		
--STATEMENT 15: Insert Statement completed. 525180  rows processed. 
		
/* 2.3. Inserting AnomalyTL50 flags */ 
INSERT	
INTO	onboarding.event_list
SELECT	cmusno
		, '26014' 				as event_id
		, tarih 				as event_date
		, 'FUND_TRANSFER' 		as domain_type
		, 'OutgoingFundTrf' 	as Event1
		, 'AnomalyTL50pctMax' 	as Event2
		, '' 					as Event3
		, '' 					as chn_name
FROM	(
	SELECT * 
	FROM komtas.ham 
	WHERE yil*ay <> 2015*10 
		AND tutar >= 1.5 * esik
		AND tutar < 2 * esik
		) a50max;
		
--STATEMENT 16: Insert Statement completed. 415250  rows processed.
		
/* 2.4. Inserting AnomalyTL100 flags */ 
INSERT	
INTO	onboarding.event_list
SELECT	cmusno
		, '26015' 				as event_id
		, tarih 				as event_date
		, 'FUND_TRANSFER' 		as domain_type
		, 'OutgoingFundTrf' 	as Event1
		, 'AnomalyTL100pctMax' 	as Event2
		, '' 					as Event3
		, '' 					as chn_name
FROM	(
	SELECT * 
	FROM komtas.ham 
	WHERE yil*ay <> 2015*10 
		AND tutar >= 2 * esik
		) a100max;

--STATEMENT 17: Insert Statement completed. 941385  rows processed.		
		
DROP TABLE IF EXISTS komtas.aylik; -- clearence.
DROP TABLE IF EXISTS komtas.ham;   -- clearence.